@@ -1,15 +1,14 @@
 #------------------------------------------------------------------------------------
 # This module contains all the ui functions for configuring Point objects
+#------------------------------------------------------------------------------------
 #
 # External API functions intended for use by other editor modules:
-#    edit_point - Open the edit point wtop level window
+#    edit_point - Open the edit point top level window
 #
 # Makes the following external API calls to other editor modules:
+#    objects.update_object(obj_id,new_obj) - Update the configuration on save
 #    objects.point_exists(point_id) - To see if a specified point ID exists
-#    objects.delete_point_object(object_id, new_item_id) - Delete before redrawing (on ok/apply)
-#    objects.redraw_point_object(object_id, new_item_idRedraw after deleting (on ok/apply)
 #    objects.point(point_id) - To get the object_id for a given point_id
-#    run_layout.process_object_update(object_id) - To process updates on ok/apply
 #
 # Accesses the following external editor objects directly:
 #    objects.point_index - To iterate through all the point objects
@@ -23,14 +22,30 @@
 #    common.selection_buttons
 #    common.signal_route_selections
 #    common.window_controls
+#
 #------------------------------------------------------------------------------------
 
+import copy
+
 from tkinter import *
 from tkinter import ttk
 
 from . import common
 from . import objects
-from . import run_layout
+
+#------------------------------------------------------------------------------------
+# Function to set the read-only "switched with" parameter. This is the back-reference
+# to the point that is configured to auto-switch the current point (else zero). This
+# is only used within the UI so doesn't need to be tracked in the point object dict
+#------------------------------------------------------------------------------------
+
+def switched_with_point(object_id):
+    switched_with_point_id = 0
+    for point_id in objects.point_index:
+        also_switch_point_id = objects.schematic_objects[objects.point(point_id)]["alsoswitch"]
+        if also_switch_point_id == objects.schematic_objects[object_id]["itemid"]:
+            switched_with_point_id = int(point_id)
+    return(switched_with_point_id)
 
 #------------------------------------------------------------------------------------
 # Function to load the initial UI state when the Edit window is created
@@ -44,6 +59,7 @@ def load_state(point):
     # Set the Initial UI state from the current object settings
     point.config.pointid.set_value(objects.schematic_objects[object_id]["itemid"])
     point.config.alsoswitch.set_value(objects.schematic_objects[object_id]["alsoswitch"])
+    point.config.alsoswitch.set_switched_with(switched_with_point(object_id))
     point.config.pointtype.set_value(objects.schematic_objects[object_id]["itemtype"])
     # These are the general settings for the point
     auto = objects.schematic_objects[object_id]["automatic"]
@@ -74,29 +90,25 @@ def save_state(point, close_window:bool):
     # been validated on entry, but changes to other objects may have been made since then
     elif (point.config.pointid.validate() and point.config.alsoswitch.validate() and
              point.config.settings.validate() and point.config.dccsettings.validate()):
-        # Get the Point ID (this may or may not have changed) - Note that we don't save  
-        # the value to the dictionary - instead we pass to the update point function
-        new_id = point.config.pointid.get_value()
-        # Update the point coniguration from the current user selections
-        objects.schematic_objects[object_id]["itemtype"] = point.config.pointtype.get_value()
-        objects.schematic_objects[object_id]["alsoswitch"] = point.config.alsoswitch.get_value()
+        # Copy the original point Configuration (elements get overwritten as required)
+        new_object_configuration = copy.deepcopy(objects.schematic_objects[object_id])
+        # Update the point coniguration elements from the current user selections
+        new_object_configuration["itemid"] = point.config.pointid.get_value()
+        new_object_configuration["itemtype"] = point.config.pointtype.get_value()
+        new_object_configuration["alsoswitch"] = point.config.alsoswitch.get_value()
         # These are the general settings
         rot, rev, auto, fpl = point.config.settings.get_values()
-        objects.schematic_objects[object_id]["reverse"] = rev
-        objects.schematic_objects[object_id]["automatic"] = auto
-        objects.schematic_objects[object_id]["hasfpl"] = fpl
-        if rot: objects.schematic_objects[object_id]["orientation"] = 180
-        else: objects.schematic_objects[object_id]["orientation"] = 0
-        # Get the  DCC address - note that dcc.get_value returns [address,state]
-        # in this instance we only need the DCC address element(element [0])
-        add, rev = point.config.dccsettings.get_values ()
-        objects.schematic_objects[object_id]["dccaddress"] = add
-        objects.schematic_objects[object_id]["dccreversed"] = rev
-        # Delete the point object from the canvas and redraw in its new configuration
-        objects.delete_point_object(object_id)
-        objects.redraw_point_object(object_id, item_id = new_id)
-        # Process any layout changes (interlocking etc)
-        run_layout.process_object_update(object_id)
+        new_object_configuration["reverse"] = rev
+        new_object_configuration["automatic"] = auto
+        new_object_configuration["hasfpl"] = fpl
+        if rot: new_object_configuration["orientation"] = 180
+        else: new_object_configuration["orientation"] = 0
+        # Get the  DCC address
+        add, rev = point.config.dccsettings.get_values()
+        new_object_configuration["dccaddress"] = add
+        new_object_configuration["dccreversed"] = rev
+        # Save the updated configuration (and re-draw the object)
+        objects.update_object(object_id, new_object_configuration)
         # Close window on "OK" or re-load UI for "apply"
         if close_window: point.window.destroy()
         else: load_state(point)
@@ -117,8 +129,9 @@ def save_state(point, close_window:bool):
 #    "set_value" - will set the current value of the entry box (int)
 #    "get_value" - will return the last "valid" value of the entry box (int)
 #    "get_initial_value" - will return the original value of the entry box (int)
-# Class instance methods overridden by this class are:
+# Class instance methods provided by this class are:
 #    "validate" - Also validate the point is automatic and not switched by another point
+#    "set_switched_with" - to set the read-only value for the "switched_with" point
 #------------------------------------------------------------------------------------
 
 class also_switch_selection(common.int_item_id_entry_box):
@@ -128,13 +141,23 @@ class also_switch_selection(common.int_item_id_entry_box):
         exists_function = objects.point_exists
         current_id_function = parent_object.pointid.get_value
         # Create the Label Frame for the "also switch" entry box
-        self.frame = LabelFrame(parent_frame, text="ID of point to 'Also Switch'")
+        self.frame = LabelFrame(parent_frame, text="Automatic switching")
         # Call the common base class init function to create the EB
-        super().__init__(self.frame, tool_tip = "Enter the ID of an existing fully "+
-                "automatic point to be switched with this point (or leave blank)",
+        self.label1 = Label(self.frame,text="Switch point:")
+        self.label1.pack(side=LEFT, padx=2, pady=2)
+        super().__init__(self.frame, tool_tip = "Enter the ID of another (fully "+
+                "automatic) point to be switched with this point (or leave blank)",
                 exists_function=exists_function, current_id_function=current_id_function)
-        self.pack(padx=2, pady=2)
-        
+        self.pack(side=LEFT, padx=2, pady=2)
+        # This is the read-only element for the point this point is switched with
+        self.switched_with = StringVar(parent_frame, "")
+        self.label2 = Label(self.frame,text="Switched with:")
+        self.label2.pack(side=LEFT, padx=2, pady=2)
+        self.switched_eb = Entry(self.frame, width=3, textvariable=self.switched_with,
+                                            justify='center',state="disabled")
+        self.switched_eb.pack(side=LEFT, padx=2, pady=2)
+        self.TT1 = common.CreateToolTip(self.switched_eb, "ID of the point that "+
+                                       "will automatically switch this point")
     def validate(self):
         # Do the basic item validation first (exists and not current item ID)
         valid = super().validate(update_validation_status=False)
@@ -146,17 +169,21 @@ class also_switch_selection(common.int_item_id_entry_box):
                 valid = False
             else:
                 # Test to see if the entered point is already being autoswitched by another point
-                if self.initial_value.get() == "": initial_autoswitch = 0
-                else: initial_autoswitch = int(self.initial_value.get())
+                if self.initial_value == "": initial_autoswitch = 0
+                else: initial_autoswitch = int(self.initial_value)
                 for point_id in objects.point_index:
                     other_autoswitch = objects.schematic_objects[objects.point(point_id)]["alsoswitch"]
                     if other_autoswitch == autoswitch and autoswitch != initial_autoswitch:
                         self.TT.text = ("Point "+str(autoswitch)+" is already configured "+
-                                              "to 'also switch' with point "+point_id)
+                                              "to be switched with point "+point_id)
                         valid = False       
         self.set_validation_status(valid)
         return(valid)
 
+    def set_switched_with(self, point_id:int):
+        if point_id > 0: self.switched_with.set(str(point_id))
+        else: self.switched_with.set("")
+            
 #------------------------------------------------------------------------------------
 # Class for the General Settings UI Element.
 # Class instance methods provided by this class:
@@ -181,17 +208,17 @@ class general_settings():
                         tool_tip="Select to rotate point by 180 degrees")
         self.CB1.pack(side=LEFT, padx=2, pady=2)
         self.CB2 = common.check_box(self.subframe1, label="Facing point lock", width=16,
-                tool_tip="Select for a Facing Point Lock (not fully automatic points)")
+                tool_tip="Select to include a Facing Point Lock (manually switched points only)")
         self.CB2.pack(side=LEFT, padx=2, pady=2)
         # Create a subframe to hold the second 2 buttons
         self.subframe2 = Frame(self.frame)
         self.subframe2.pack()
         self.CB3 = common.check_box(self.subframe2, label="Reversed", width=9,
-                        tool_tip="Select to reverse the point blades")
+                        tool_tip="Select to reverse the switching logic of the point blades")
         self.CB3.pack(side=LEFT, padx=2, pady=2)
         self.CB4 = common.check_box(self.subframe2, label="Fully automatic", width=16,
-            tool_tip="Select to enable this point to be 'also switched' "+
-                        "by another point", callback= self.automatic_updated)
+            tool_tip="Select to create the point without manual controls (to be switched "+
+                        "with another point)", callback= self.automatic_updated)
         self.CB4.pack(side=LEFT, padx=2, pady=2)
 
     def automatic_updated(self):
@@ -209,7 +236,7 @@ class general_settings():
             for point_id in objects.point_index:
                 other_autoswitch = objects.schematic_objects[objects.point(point_id)]["alsoswitch"]
                 if other_autoswitch == self.parent_object.pointid.get_initial_value():
-                    self.CB4.TT.text = ("Point is configured to be 'also switched' by point " +
+                    self.CB4.TT.text = ("Point is configured to switch with point " +
                                            point_id + " so must remain 'fully automatic'")
                     self.CB4.config(fg="red")
                     valid = False
@@ -268,7 +295,6 @@ class dcc_address_settings(common.dcc_entry_box):
         self.entry_updated()
         
     def get_values(self):
-        # Note that we only need the address element from the dcc entry box
         return (self.EB.get_value(), self.CB.get_value())
     
 #------------------------------------------------------------------------------------
@@ -332,12 +358,12 @@ class signal_route_interlocking_frame():
             for sig_interlocking_routes in sig_interlocking_frame:
                 # sig_interlocking_routes comprises [sig_id, [main, lh1, lh2, rh1, rh2]]
                 # Where each route element is a boolean value (True or False)            
-                self.sigelements.append(common.signal_route_selections(self.subframe, read_only=True,
-                                    tool_tip="Edit the appropriate signals\nto configure interlocking"))
+                self.sigelements.append(common.signal_route_selections(self.subframe,read_only=True,
+                        tool_tip="Edit the appropriate signals\nto configure interlocking"))
                 self.sigelements[-1].frame.pack()
                 self.sigelements[-1].set_values (sig_interlocking_routes)
         else:
-            self.label = Label(self.subframe, text="Edit the appropriate signals\nto configure interlocking")
+            self.label = Label(self.subframe, text="No interlocked signals")
             self.label.pack()
 
 #------------------------------------------------------------------------------------
@@ -359,8 +385,12 @@ class edit_point():
         # Creatre the basic Top Level window
         self.window = Toplevel(root)
         self.window.attributes('-topmost',True)
-        # Create the Window tabs
+        # Create the Notebook (for the tabs) 
         self.tabs = ttk.Notebook(self.window)
+        # When you change tabs tkinter focuses on the first entry box - we don't want this
+        # So we bind the tab changed event to a function which will focus on something else 
+        self.tabs.bind ('<<NotebookTabChanged>>', self.tab_changed)
+        # Create the Window tabs
         self.tab1 = Frame(self.tabs)
         self.tabs.add(self.tab1, text="Configration")
         self.tab2 = Frame(self.tabs)
@@ -375,4 +405,10 @@ class edit_point():
         # load the initial UI state
         load_state(self)
 
+    def tab_changed(self,event):
+        # Focus on the top level window to remove focus from the first entry box
+        # THIS IS STILL NOT WORKING AS IT LEAVES THE ENTRY BOX HIGHLIGHTED
+        # self.window.focus()
+        pass
+
 #############################################################################################
